/* GIMP image attachment browser plug-in
 * Copyright (C) 2004 William Skaggs
 * All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of the Author of the
 * Software shall not be used in advertising or otherwise to promote the
 * sale, use or other dealings in this Software without prior written
 * authorization from the Author.
 */

#include "config.h"
#include <stdio.h>
#include <string.h>

#include <libgimp/gimp.h>
#include <libgimp/gimpui.h>

#include "plugin-intl.h"


/*  Constants  */

#define PROCEDURE_NAME   "meta_data"

#define DATA_KEY_VALS    "meta_data"

#define PARASITE_KEY     "meta-data-options"

typedef struct
{
  gint32      image_id;
  GimpParasite *parasite;
  gboolean    item_is_text;
  gboolean    editable;
  gboolean    modified;
  gchar      *name;
  gboolean    dialog_built;
  const gchar *fname;
} PlugInVals;

typedef struct
{
  gint32    image_id;
} PlugInImageVals;

typedef struct
{
  gint32    drawable_id;
} PlugInDrawableVals;


enum
  {
    NAME_COLUMN,
    N_COLUMNS
  };

/*  Local function prototypes  */

static void   query (void);
static void   run (const gchar         *name,
		   gint                nparams,
		   const GimpParam     *param,
		   gint                *nreturn_vals,
		   GimpParam           **return_vals);
static gboolean dialog (GimpDrawable        *drawable,
			PlugInImageVals     *image_vals,
			PlugInDrawableVals  *drawable_vals);
static void  parasite_list_changed_cb (GtkTreeSelection *selection, 
				       gpointer data);
static void  save_parasite_cb (GtkButton *button, 
			       gpointer data);
static void  delete_parasite_cb (GtkButton *button, 
				 gpointer data);
static void  load_parasite_from_file_cb (GtkButton *button, 
					 gpointer data);
static void  load_icc_profile_cb (GtkButton *button, 
					 gpointer data);
static void  save_parasite_to_file_cb (GtkButton *button, 
				       gpointer data);
static void  reset_cb (GtkButton *button, 
		       gpointer data);
static void  new_parasite_cb (GtkWidget *widget, 
			      gpointer data);
static gboolean ask_question (const gchar *question);
static void  text_changed_cb (GtkTextBuffer *buffer, 
			      gpointer data);
static void  flags_changed_cb (GtkToggleButton *button, 
			       gpointer data);
static void tree_model_get_iter_last (GtkTreeModel *model,
				      GtkTreeIter *iter);
static void set_modified (gboolean modified);
static void store_filename (GtkWidget *widget, 
			    gpointer data);
static void activate_dialog_entry_cb (GtkEntry *entry,
				      gpointer data);

/*  Local variables  */

const PlugInVals default_vals =
{
  0,
  NULL,
  FALSE,
  FALSE,
  FALSE,
  NULL,
  FALSE,
  ""
};

const PlugInImageVals default_image_vals =
{
  0
};

const PlugInDrawableVals default_drawable_vals =
{
  0
};

static PlugInVals         vals;
static PlugInImageVals    image_vals;
static PlugInDrawableVals drawable_vals;
static GtkWidget  *text_view = NULL;
static GtkWidget  *name_label = NULL;
static GtkWidget *persistence_checkbutton = NULL;
static GtkWidget *metadata_dlg;
static GtkWidget *apply_button;
static GtkWidget *reset_button;
static gchar    **parasites = NULL;
static GtkListStore *parasite_list_store; 
static GtkTreeSelection *parasite_list_selection;
static PangoAttrList *name_attrlist;


GimpPlugInInfo PLUG_IN_INFO =
{
  NULL,  /* init_proc  */
  NULL,  /* quit_proc  */
  query, /* query_proc */
  run,   /* run_proc   */
};

MAIN ()

static void
query (void)
{
  gchar *help_path;
  gchar *help_uri;

  static GimpParamDef args[] =
  {
    { GIMP_PDB_INT32,    "run_mode",   "Interactive, non-interactive"    },
    { GIMP_PDB_IMAGE,    "image",      "Input image"                     },
    { GIMP_PDB_DRAWABLE, "drawable",   "Input drawable"                  }
  };

  gimp_plugin_domain_register (PLUGIN_NAME, LOCALEDIR);

  help_path = g_build_filename (DATADIR, "help", NULL);
  help_uri = g_filename_to_uri (help_path, NULL, NULL);
  g_free (help_path);

  gimp_plugin_help_register ("http://developer.gimp.org/plug-in-metadata/help",
                             help_uri);

  gimp_install_procedure (PROCEDURE_NAME,
			  "View and edit image attachments.",
			  "This is an image attachment (parasite) browser.  It is interactive only.  It can be used to list attachments, view the contents if they are text, modify them, create new ones, delete existing ones, save the contents to a file, or load the contents of a file as a new attachment.",
			  "William Skaggs",
			  "William Skaggs",
			  "2000-2004",
			  N_("<Image>/Filters/Generic/Attachments..."),
			  "RGB*, GRAY*, INDEXED*",
			  GIMP_PLUGIN,
			  G_N_ELEMENTS (args), 0,
			  args, NULL);
}

static void
run (const gchar      *name,
     gint              n_params,
     const GimpParam  *param,
     gint             *nreturn_vals,
     GimpParam       **return_vals)
{
  static GimpParam   values[1];
  GimpDrawable      *drawable;
  gint32             image_ID;
  GimpRunMode        run_mode;
  GimpPDBStatusType  status = GIMP_PDB_SUCCESS;

  *nreturn_vals = 1;
  *return_vals  = values;


  /*  Initialize i18n support  */
  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
#ifdef HAVE_BIND_TEXTDOMAIN_CODESET
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
#endif
  textdomain (GETTEXT_PACKAGE);

  run_mode = param[0].data.d_int32;
  image_ID = param[1].data.d_int32;
  drawable = gimp_drawable_get (param[2].data.d_drawable);

  /*  Initialize with default values  */
  vals          = default_vals;
  image_vals    = default_image_vals;
  drawable_vals = default_drawable_vals;

  vals.image_id = image_ID;

  if (strcmp (name, PROCEDURE_NAME) == 0)
    {
      switch (run_mode)
	{
	case GIMP_RUN_INTERACTIVE:
	case GIMP_RUN_WITH_LAST_VALS:
	  /*  Possibly retrieve data  */
	  gimp_get_data (DATA_KEY_VALS,    &vals);

	  if (! dialog (drawable, &image_vals, &drawable_vals))
	    {
	      status = GIMP_PDB_CANCEL;
	    }
	  break;

	default:
	  break;
	}
    }
  else
    {
      status = GIMP_PDB_CALLING_ERROR;
    }

  values[0].type = GIMP_PDB_STATUS;
  values[0].data.d_status = status;
}


gboolean
dialog (GimpDrawable       *drawable,
	PlugInImageVals    *image_vals,
	PlugInDrawableVals *drawable_vals)
{
  GtkWidget *dlg;
  GtkWidget *main_hbox;
  GtkWidget *main_vbox;
  GtkWidget *frame;
  GtkTreeIter iter;
  GtkWidget *tree;
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;
  GtkWidget *scroll;
  GtkWidget *label;
  GtkWidget *hbox;
  GtkWidget *widget;
  GtkWidget *button;
  GtkWidget *menubar;
  GtkWidget *menuitem;
  GtkWidget *menu;
  GtkTextBuffer *buffer;
  PangoAttribute *attr;
  gboolean   run = FALSE;
  gint       num_parasites;
  gint       i;

  gimp_ui_init (PLUGIN_NAME, TRUE);

  metadata_dlg = dlg = gimp_dialog_new (_("Image Attachments"), PLUGIN_NAME,
					NULL, 0,
					gimp_standard_help_func, "plug-in-metadata",
					
					GTK_STOCK_OK,     GTK_RESPONSE_OK,
					
					NULL);

  menubar = gtk_menu_bar_new ();
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), menubar, FALSE, FALSE, 0);
  gtk_widget_show (menubar);

  /* File menu */
  menuitem = gtk_menu_item_new_with_mnemonic ("_File");
  gtk_menu_shell_append (GTK_MENU_SHELL (menubar), menuitem);
  gtk_widget_show (menuitem);

  menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), menu);
  gtk_widget_show (menu);

  menuitem = gtk_menu_item_new_with_mnemonic ("_Load from file");
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
  g_signal_connect (G_OBJECT (menuitem), "activate",
		    G_CALLBACK (load_parasite_from_file_cb),
		    NULL);
  gtk_widget_show (menuitem);

  menuitem = gtk_menu_item_new_with_mnemonic ("_Save to file");
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
  g_signal_connect (G_OBJECT (menuitem), "activate",
		    G_CALLBACK (save_parasite_to_file_cb),
		    NULL);
  gtk_widget_show (menuitem);

  menuitem = gtk_menu_item_new_with_mnemonic ("_Load ICC Profile");
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
  g_signal_connect (G_OBJECT (menuitem), "activate",
		    G_CALLBACK (load_icc_profile_cb),
		    NULL);
  gtk_widget_show (menuitem);

  /* Edit menu */
  menuitem = gtk_menu_item_new_with_mnemonic ("_Edit");
  gtk_menu_shell_append (GTK_MENU_SHELL (menubar), menuitem);
  gtk_widget_show (menuitem);

  menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), menu);
  gtk_widget_show (menu);

  menuitem = gtk_menu_item_new_with_mnemonic (_("_Delete"));
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
  g_signal_connect (G_OBJECT (menuitem), "activate",
		    G_CALLBACK (delete_parasite_cb),
		    NULL);
  gtk_widget_show (menuitem);

  menuitem = gtk_menu_item_new_with_mnemonic (_("_New Item"));
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
  g_signal_connect (G_OBJECT (menuitem), "activate",
		    G_CALLBACK (new_parasite_cb),
		    NULL);
  gtk_widget_show (menuitem);

  frame = gtk_frame_new (gimp_image_get_name (vals.image_id));
  gtk_container_set_border_width (GTK_CONTAINER (frame), 6);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), frame, FALSE, FALSE, 0);
  gtk_widget_show (frame);

  main_hbox = gtk_hbox_new (FALSE, 4);
  gtk_container_set_border_width (GTK_CONTAINER (main_hbox), 6);
  gtk_container_add (GTK_CONTAINER (frame), main_hbox);
  gtk_widget_show (main_hbox);

  main_vbox = gtk_vbox_new (FALSE, 4);
  gtk_box_pack_start (GTK_BOX (main_hbox), main_vbox, FALSE, FALSE, 5);
  gtk_widget_show (main_vbox);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, FALSE, 10);
  gtk_widget_show (hbox);

  label = gtk_label_new (gimp_image_get_filename (vals.image_id));
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_widget_show (label);

  scroll = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_set_size_request (scroll, 200, 200);
  gtk_box_pack_start (GTK_BOX (main_vbox), scroll, TRUE, TRUE, 0);
  gtk_widget_show (scroll);

  gimp_image_parasite_list (vals.image_id, &num_parasites, &parasites);
  parasite_list_store = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING);
  for (i = 0; i < num_parasites; i++)
    {
      gtk_list_store_append (parasite_list_store, &iter);
      gtk_list_store_set (parasite_list_store, &iter,
			  NAME_COLUMN, parasites[i],
			  -1);
    }
  tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (parasite_list_store));
  gtk_container_add (GTK_CONTAINER (scroll), tree); 
  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes (_("Attachment Name"),
						     renderer,
						     "text", NAME_COLUMN,
						     NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
  parasite_list_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
  gtk_tree_selection_set_mode (parasite_list_selection, GTK_SELECTION_SINGLE);
  g_signal_connect (G_OBJECT (parasite_list_selection), "changed",
		    G_CALLBACK (parasite_list_changed_cb),
		    GINT_TO_POINTER (TRUE));
  gtk_widget_show (tree);

  widget = gtk_vseparator_new ();
  gtk_box_pack_start (GTK_BOX (main_hbox), widget, FALSE, FALSE, 5);
  gtk_widget_show (widget);

  main_vbox = gtk_vbox_new (FALSE, 4);
  gtk_box_pack_start (GTK_BOX (main_hbox), main_vbox, TRUE, TRUE, 5);
  gtk_widget_show (main_vbox);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, FALSE, 5);
  gtk_widget_show (hbox);

  label = gtk_label_new (_("Name:"));
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_widget_show (label);

  name_label = gtk_label_new ("");
  gtk_box_pack_start (GTK_BOX (hbox), name_label, FALSE, FALSE, 5);
  name_attrlist = pango_attr_list_new ();
  attr = pango_attr_underline_new (PANGO_UNDERLINE_SINGLE);
  attr->start_index = 0;
  attr->end_index = G_MAXINT;
  pango_attr_list_insert (name_attrlist, attr);
  attr = pango_attr_style_new (PANGO_STYLE_ITALIC);
  attr->start_index = 0;
  attr->end_index = G_MAXINT;
  pango_attr_list_insert (name_attrlist, attr);
  gtk_label_set_use_markup (GTK_LABEL (name_label), FALSE);
  gtk_label_set_use_underline (GTK_LABEL (name_label), FALSE);
  gtk_label_set_attributes (GTK_LABEL (name_label), name_attrlist);
  gtk_widget_show (name_label);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, FALSE, 2);
  gtk_widget_show (hbox);

  button = persistence_checkbutton = gtk_check_button_new_with_label (_("Save with XCF file"));
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 5);
  g_signal_connect (G_OBJECT (button), "toggled",
		    G_CALLBACK (flags_changed_cb),
		    NULL);
  gtk_widget_show (button);

  scroll = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_set_size_request (scroll, 300, 200);
  gtk_box_pack_start (GTK_BOX (main_vbox), scroll, TRUE, TRUE, 0);
  gtk_widget_show (scroll);

  text_view = gtk_text_view_new ();
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW(text_view), GTK_WRAP_CHAR);
  gtk_container_add (GTK_CONTAINER (scroll), text_view); 
  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
  g_signal_connect (G_OBJECT (buffer), "changed",
		    G_CALLBACK (text_changed_cb),
		    NULL);
  gtk_widget_show (text_view);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_end (GTK_BOX (main_vbox), hbox, FALSE, FALSE, 10);
  gtk_widget_show (hbox);

  button = apply_button = gtk_button_new_with_mnemonic (_("_Apply Changes"));
  gtk_widget_set_sensitive (button, FALSE);
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 5);
  g_signal_connect (G_OBJECT (button), "pressed",
		    G_CALLBACK (save_parasite_cb),
		    NULL);
  gtk_widget_show (button);

  button = reset_button = gtk_button_new_with_mnemonic (_("_Reset"));
  gtk_widget_set_sensitive (button, FALSE);
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 5);
  g_signal_connect (G_OBJECT (button), "pressed",
		    G_CALLBACK (reset_cb),
		    NULL);
  gtk_widget_show (button);


  gtk_widget_show (dlg);
  vals.dialog_built = TRUE;
  parasite_list_changed_cb (parasite_list_selection,
			    GINT_TO_POINTER (FALSE));
  run = (gimp_dialog_run (GIMP_DIALOG (dlg)) == GTK_RESPONSE_OK);

  if (vals.modified)
    if (ask_question (_("You made changes.  Do you want to apply them before quitting?")))
      save_parasite_cb (NULL, NULL);

  return run;
}


/*  Private functions  */

static void
parasite_list_changed_cb (GtkTreeSelection *selection, 
			  gpointer data)
{
  GtkTreeIter iter;
  GtkTreeModel *model;
  GtkTextBuffer *buffer = NULL;
  GimpParasite *parasite;
  gchar *name;
  gconstpointer pdata;
  gint length;
  const gchar *end;
  gint verify = GPOINTER_TO_INT (data);

  if (!vals.dialog_built)
    return;


  if (verify)
    {
      if (vals.modified)
	if (ask_question ("You made changes.  Do you want to apply them?"))
	  save_parasite_cb (NULL, NULL);
    }

  if (gtk_tree_selection_get_selected (selection, &model, &iter))
    {
      gtk_tree_model_get (model, &iter, NAME_COLUMN, &name, -1);
      vals.parasite = parasite = gimp_image_parasite_find (vals.image_id, name);

      vals.name = (gchar *)g_realloc (vals.name, strlen (name) * sizeof (gchar)); 
      strcpy (vals.name, name);

      length = gimp_parasite_data_size (parasite);
      pdata = gimp_parasite_data (parasite);

      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (persistence_checkbutton),
				    gimp_parasite_is_persistent (parasite));
      gtk_label_set_text (GTK_LABEL (name_label), name);
      buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
      if (pdata == NULL)
	{
	  gtk_text_buffer_set_text (buffer, "(NULL)", -1);
	}
      else
	{
	  if (g_utf8_validate (pdata, length, &end))
	      gtk_text_buffer_set_text (buffer, pdata, length);
	  else if (g_utf8_validate (pdata, length-1, &end))
	      gtk_text_buffer_set_text (buffer, pdata, length-1);
	  else
	    gtk_text_buffer_set_text (buffer, _("(binary data)"), -1);
	}

      g_free (name);
      set_modified (FALSE);
    }
}


static void
reset_cb (GtkButton *button,
	  gpointer data)
{
  parasite_list_changed_cb (parasite_list_selection, GINT_TO_POINTER (FALSE));
}


static void
save_parasite_cb (GtkButton *button, 
		  gpointer data)
{
  gchar *name;
  GimpParasite *parasite;
  gboolean persistent;
  GtkTextBuffer *buffer;
  GtkTextIter start, end;
  gchar *text;
  gint  size;
  gint32 flags = 0;
  const gchar *endptr;

  name = vals.name;
  if (!name || strlen (name) == 0)
    return;

  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
  gtk_text_buffer_get_bounds (buffer, &start, &end);
  text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
  size = strlen (text);
  if (size == 0)
    {
      g_message ("Cannot save an empty item.\n");
      return;
    }


  persistent = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (persistence_checkbutton));

  /* make sure text is ascii */

  parasite = gimp_image_parasite_find (vals.image_id, name);
  
  if (parasite)
    {
      if (parasite->size > 1 &&
	  !g_utf8_validate (gimp_parasite_data (parasite), parasite->size-1, &endptr))
	{
	  g_message ("This item contains binary data and cannot be modified.");
	  return;
	}
      gimp_image_parasite_detach (vals.image_id, name);
      gimp_parasite_free (parasite);
    }

  if (persistent)
    flags |= GIMP_PARASITE_PERSISTENT;
  
  gimp_image_attach_new_parasite (vals.image_id, name, flags, size, text);
  set_modified (FALSE);
}


static void
new_parasite_cb (GtkWidget *widget,
		 gpointer data)
{
  const gchar *name;
  GtkTreeIter iter;
  GtkWidget *dlg;
  GtkWidget *label;
  GtkWidget *entry;
  gint32     flags = 0;
  gint       size;
  GtkTextBuffer *buffer;
  gint       result;

  dlg = gtk_dialog_new_with_buttons ("Create Attachment",
				     GTK_WINDOW (metadata_dlg), 
				     GTK_DIALOG_DESTROY_WITH_PARENT,
				     GTK_STOCK_OK, GTK_RESPONSE_OK,
				     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				     NULL);
  label = gtk_label_new ("Name of new attachment:");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), label, FALSE, FALSE, 0);
  gtk_widget_show (label);

  entry = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), entry, TRUE, TRUE, 0);
  gtk_widget_show (entry);

  result = gtk_dialog_run (GTK_DIALOG (dlg));

  if (result == GTK_RESPONSE_OK)
    {
      name = gtk_entry_get_text (GTK_ENTRY (entry));
      if (strlen (name) == 0)
	{
	  g_error ("Cannot create attachment without name.");
	}
      else
	{
	  /* need to check if name is valid ascii */

	  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
	  gtk_text_buffer_set_text (buffer, "", -1);
	  size = strlen ("");
	  set_modified (FALSE);

	  gimp_image_attach_new_parasite (vals.image_id, name, flags, size, "");
	  
	  gtk_list_store_append (parasite_list_store, &iter);
	  gtk_list_store_set (parasite_list_store, &iter,
			      NAME_COLUMN, name,
			      -1);
	  gtk_tree_selection_select_iter (parasite_list_selection, &iter);
	}
    }

  gtk_widget_destroy (label);
  gtk_widget_destroy (entry);
  gtk_widget_destroy (dlg);
}


static void
delete_parasite_cb (GtkButton *button, 
		    gpointer data)
{
  gchar *name;
  GimpParasite *parasite;
  gchar msg[50];
  GtkTreeModel *model;
  GtkTreeIter iter;
  gboolean valid;

  name = vals.name;
  if (!name || strlen (name) == 0)
    {
      g_message ("No name or name is null");
      return;
    }

  parasite = gimp_image_parasite_find (vals.image_id, name);
  gtk_tree_selection_get_selected (parasite_list_selection, &model, &iter);

  if (!parasite)
    {
      g_message ("Unable to delete attachment: not found");
      fprintf (stderr, "Name: %s\n", name);
      return;
    }

  if (parasite)
    {
      sprintf (msg, "Really delete attachment '%s'?", name);
      if (!ask_question (msg))
	return;
      gimp_image_parasite_detach (vals.image_id, name);
      gimp_parasite_free (parasite);
      valid = gtk_list_store_remove (parasite_list_store, &iter);
      /*if (!valid)
	tree_model_get_iter_last (model, &iter);*/
      set_modified (FALSE);
      if (valid)
	gtk_tree_selection_select_iter (parasite_list_selection, &iter);
      /* parasite_list_changed_cb (parasite_list_selection, NULL); */
    }
}


static void
tree_model_get_iter_last (GtkTreeModel *model,
			  GtkTreeIter *iter)
{
  gint n = 0;

  gtk_tree_model_get_iter_first (model, iter);

  while (gtk_tree_model_iter_next (model, iter))
    n++;

  if (n >= 1)
    gtk_tree_model_iter_nth_child (model, iter, NULL, n-1);
}

    
static gboolean
ask_question (const gchar *question)
{
  gboolean response;
  GtkWidget *popup;

  popup = gtk_message_dialog_new (GTK_WINDOW (metadata_dlg), 
				  GTK_DIALOG_DESTROY_WITH_PARENT,
				  GTK_MESSAGE_QUESTION, 
				  GTK_BUTTONS_YES_NO,
				  question);
  response = gtk_dialog_run (GTK_DIALOG (popup));
  gtk_widget_destroy (popup);

  if (response == GTK_RESPONSE_YES)
    return (TRUE);
  else
    return (FALSE);
}

static void
text_changed_cb (GtkTextBuffer *buffer, 
		 gpointer data)
{
  set_modified (TRUE);
}

static void
flags_changed_cb (GtkToggleButton *button, 
		 gpointer data)
{
  set_modified (TRUE);
}


static void
set_modified (gboolean modified)
{
  vals.modified = modified;
  gtk_widget_set_sensitive (apply_button, modified);
  gtk_widget_set_sensitive (reset_button, modified);
}


static void
load_parasite_from_file_cb (GtkButton *button,
			    gpointer user_data)
{
  GtkWidget *file_selector;
  GtkWidget *dlg;
  GtkWidget *label;
  gint response;
  GtkWidget *entry;
  gchar *name;
  const gchar *tmpname;
  gchar *data;
  gint size;
  FILE *file;
  gint32 flags;
  GtkTreeIter iter;

  dlg = gtk_dialog_new_with_buttons ("Create Attachment",
				     GTK_WINDOW (metadata_dlg), 
				     GTK_DIALOG_DESTROY_WITH_PARENT,
				     GTK_STOCK_OK, GTK_RESPONSE_OK,
				     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				     NULL);
  label = gtk_label_new ("Name of new attachment:");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), label, FALSE, FALSE, 0);
  gtk_widget_show (label);

  entry = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), entry, TRUE, TRUE, 0);
  g_signal_connect (GTK_OBJECT (entry), "activate",
		    G_CALLBACK (activate_dialog_entry_cb),
		    (gpointer) dlg);
  gtk_widget_show (entry);

  response = gtk_dialog_run (GTK_DIALOG (dlg));
  tmpname = gtk_entry_get_text (GTK_ENTRY (entry));
  name = (gchar*)g_malloc((1 + strlen (tmpname))*sizeof (gchar));
  strcpy (name, tmpname);
  gtk_widget_destroy (label);
  gtk_widget_destroy (entry);
  gtk_widget_destroy (dlg);

  if (response != GTK_RESPONSE_OK)
    return;

  if (strlen (name) == 0)
    {
      g_message ("Cannot create attachment without name.");
    }
  /* need to check if name is valid ascii */
  /* need to make sure there is not already an attachment with this name */

  file_selector = gtk_file_selection_new (_("Name of file containing attachment data:"));
  g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
		    "clicked",
		    G_CALLBACK (store_filename),
		    (gpointer) file_selector);
  response = gtk_dialog_run (GTK_DIALOG (file_selector));
  gtk_widget_destroy (file_selector);

  if (response != GTK_RESPONSE_OK)
    return;
  
  file = fopen (vals.fname, "rb");
  if (!file)
    {
      g_message ("Cannot open file '%s' for reading.", vals.fname);
      return;
    }
  
  size = 0;
  while (getc (file) != EOF)
    size++;
  rewind (file);
  data = (gchar*)g_malloc (size * sizeof (gchar));
  fread (data, sizeof (gchar), size, file);
  fclose (file);
  
  flags = GIMP_PARASITE_PERSISTENT;
  gimp_image_attach_new_parasite (vals.image_id, name, flags, size, data);
  gtk_list_store_append (parasite_list_store, &iter);
  gtk_list_store_set (parasite_list_store, &iter,
		      NAME_COLUMN, name,
		      -1);
  gtk_tree_selection_select_iter (parasite_list_selection, &iter);
  set_modified (FALSE);
  g_free (data);
  g_free (name);
}


static void
load_icc_profile_cb (GtkButton *button,
		     gpointer user_data)
{
  GtkWidget *file_selector;
  gint response;
  gchar *data;
  gint size;
  FILE *file;
  gint32 flags;
  GtkTreeIter iter;


  file_selector = gtk_file_selection_new (_("File name of ICC Profile:"));
  g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
		    "clicked",
		    G_CALLBACK (store_filename),
		    (gpointer) file_selector);
  response = gtk_dialog_run (GTK_DIALOG (file_selector));
  gtk_widget_destroy (file_selector);

  if (response != GTK_RESPONSE_OK)
    return;
  
  file = fopen (vals.fname, "rb");
  if (!file)
    {
      g_message ("Cannot open file '%s' for reading.", vals.fname);
      return;
    }
  
  size = 0;
  while (getc (file) != EOF)
    size++;
  rewind (file);
  data = (gchar*)g_malloc (size * sizeof (gchar));
  fread (data, sizeof (gchar), size, file);
  fclose (file);
  
  flags = GIMP_PARASITE_PERSISTENT;
  gimp_image_attach_new_parasite (vals.image_id, "icc-profile", flags, size, data);
  gtk_list_store_append (parasite_list_store, &iter);
  gtk_list_store_set (parasite_list_store, &iter,
		      NAME_COLUMN, "icc-profile",
		      -1);
  gtk_tree_selection_select_iter (parasite_list_selection, &iter);
  set_modified (FALSE);
  g_free (data);
}


static void activate_dialog_entry_cb (GtkEntry *entry,
				      gpointer data)
{
  GtkDialog *dlg = (GtkDialog*)data;

  gtk_dialog_response (dlg, GTK_RESPONSE_OK);
}
  

static void store_filename (GtkWidget *widget, 
			    gpointer data) 
{
  GtkWidget *file_selector = (GtkWidget *)data;

   vals.fname = gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector));
}


static void
save_parasite_to_file_cb (GtkButton *button,
			    gpointer user_data)
{
  GtkWidget *file_selector;
  gint response;
  gchar *name;
  const gchar *data;
  glong size;
  FILE *file;
  GimpParasite *parasite;

  name = vals.name;
  if (!name || strlen (name) == 0)
    return;

  file_selector = gtk_file_selection_new (_("File name"));
  g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
		    "clicked",
		    G_CALLBACK (store_filename),
		    (gpointer) file_selector);
  response = gtk_dialog_run (GTK_DIALOG (file_selector));
  gtk_widget_destroy (file_selector);

  if (response != GTK_RESPONSE_OK)
    return;
  
  file = fopen (vals.fname, "wb");
  if (!file)
    {
      g_message ("Cannot open file '%s' for writing.", vals.fname);
      return;
    }
  
  parasite = gimp_image_parasite_find (vals.image_id, name);
  data = gimp_parasite_data (parasite);
  size = gimp_parasite_data_size (parasite);
  if (size != fwrite(data, 1, size, file))
    {
      g_message ("Error writing file '%s'", vals.fname);
    }
  fclose (file);

}

